;**********************************************************
;         MATH32: 32 bit math macro collection
;                  For the PIC16 family
;
;Written by: Marco DI LEO <m.dileo@sistinf.it>
;Date: 1998/04/21
;Version: 1.0
;
;The following code is Copyright (C) 1998 by Marco DI LEO.
;Permission is hereby granted to use the code for
;non-commercial personal use providing this notice is not
;removed from source code copies. Contact me if you wish to
;use the code (or part of it) for a commercial use. 
;This source code is provided AS-IS with no warranty
;expressed or implied.
;**********************************************************

;History:
;V1.0 1998/04/21 First release.

;These routines are implemented as macros to allow to have
;maximum control over the source and destination operands.
;If code space is scarce you can create a subroutine containing
;just the macro invocation and a return.



	ifndef	_math32_inc_		;Check if file already included

	#define	_math32_inc_		;Flag that file is included


;Moves the 32 bit literal value lit in the dst area.

;The macro is optimized to save program words if some of
;the bytes of op1 are zero. In this case it produces a
;clrf instead of a movlw/movwf sequence.

mmovl32	macro	dst, lit

	local	byteop			;Temporary storage for byte value

byteop = (lit & 0x000000ff)		;Get operand LSB
	if	byteop == 0		;Null byte?
	clrf	dst			;Yes, simply clear destination byte
	else
	movlw	byteop			;No, get value in W...
	movwf	dst			;...and put it to destination
	endif

byteop = (lit & 0x0000ff00) >> 8	;Get operand LSB + 1
	if	byteop == 0
	clrf	dst+1
	else
	movlw	byteop
	movwf	dst+1
	endif

byteop = (lit & 0x00ff0000) >> 16	;Get operand LSB + 2
	if	byteop == 0
	clrf	dst+2
	else
	movlw	byteop
	movwf	dst+2
	endif

byteop = (lit & 0xff000000) >> 24	;Get operand LSB + 3 (MSB)
	if	byteop == 0
	clrf	dst+3
	else
	movlw	byteop
	movwf	dst+3
	endif

	endm


;Adds the two 32 bit values op1 and op2 and put the result in dst.

;The macro adjust itself to save space if dst is the same location
;of the op1 operand (i.e. op1 = op1 + op2). 

madd32	macro	op1, op2, dst

	movf	op2, W			;Get operand LSB
	if	dst == op1		;Destination is operand 1?
	addwf	op1, F			;Yes, add to operand 1 LSB
	else
	addwf	op1, W			;No, store sum in destination LSB
	movwf	dst
	endif

	movf	op2+1, W		;Get operand LSB + 1
	btfsc	STATUS, C		;Carry from previous byte?
	incfsz	op2+1, W		;Yes, re-get operand and increment
					;it. If result is zero skip next
					;add (we don't need to add zero!)
					;preserving the carry for next byte
	if	dst == op1
	addwf	op1+1, F		;Add to working register LSB + 1
	else
	addwf	op1+1, W
	movwf	dst+1
	endif

	movf	op2+2, W		;Get operand LSB + 2
	btfsc	STATUS, C		;See LSB + 1 for comments
	incfsz	op2+2, W
	if	dst == op1
	addwf	op1+2, F
	else
	addwf	op1+2, W
	movwf	dst+2
	endif

	movf	op2+3, W		;Get operand LSB + 3 (MSB)
	btfsc	STATUS, C		;See LSB + 1 for comments
	incfsz	op2+3, W
	if	dst == op1
	addwf	op1+3, F
	else
	addwf	op1+3, W
	movwf	dst+3
	endif

	endm


;Subtract the 32 bit value in op2 from op1 and put the result
;in dst.

;The macro adjust itself to save space if dst is the same location
;of the op1 operand (i.e. op1 = op1 - op2).

msub32	macro	op1, op2, dst

	movf	op2, W			;Get operand LSB
	if	dst == op1		;Destination is operand 1?
	subwf	op1, F			;Yes, subtract from op1 LSB
	else
	subwf	op1, W			;No, store result in destination LSB
	movwf	dst
	endif

	movf	op2+1, W		;Get operand LSB + 1
	btfss	STATUS, C		;Borrow from previous byte?
	incfsz	op2+1, W		;Yes, re-get operand and increment
					;it. If result is zero skip next
					;sub (we don't need to sub zero!)
					;preserving the carry for next byte
	if	dst == op1
	subwf	op1+1, F		;Subtract from op1 LSB + 1
	else
	subwf	op1+1, W
	movwf	dst+1
	endif

	movf	op2+2, W		;Get operand LSB + 2
	btfss	STATUS, C		;See LSB + 1 for comments
	incfsz	op2+2, W
	if	dst == op1
	subwf	op1+2, F
	else
	subwf	op1+2, W
	movwf	dst+2
	endif

	movf	op2+3, W		;Get operand LSB + 3 (MSB)
	btfss	STATUS, C		;See LSB + 1 for comments
	incfsz	op2+3, W
	if	dst == op1
	subwf	op1+3, F
	else
	subwf	op1+3, W
	movwf	dst+3
	endif

	endm


;Adds the literal 32 bit value in lit to the op1 area and put
;the result in dst.

;The macro adjust itself to save space if dst is the same location
;of the op1 operand (i.e. op1 = op1 + lit).

maddl32	macro	op1, lit, dst

	local	byteop			;Temporary storage for byte value

byteop = (lit & 0x000000ff)		;Get operand LSB
	movlw	byteop			
	if	dst == op1		;Destination is operand 1?
	addwf	op1, F			;Yes, add to working register LSB
	else
	addwf	op1, W			;No, store sum in destination LSB
	movwf	dst
	endif


byteop = (lit & 0x0000ff00) >> 8	;Get operand LSB + 1
	movlw	byteop
	btfsc	STATUS, C		;Carry from previous byte?
	movlw	byteop+1		;Yes, re-get operand and increment it.
	if	byteop+1 == 0		;If result is zero and carry set skip
	btfss	STATUS, C		;next add (we don't need to add zero!)
					;preserving the carry for next byte
	endif
	if	dst == op1
	addwf	op1+1, F		;Add to working register LSB + 1
	else
	addwf	op1+1, W
	movwf	dst+1
	endif

byteop = (lit & 0x00ff0000) >> 16	;Get operand LSB + 2
	movlw	byteop
	btfsc	STATUS, C		;See LSB + 1 for comments
	movlw	byteop+1
	if	byteop+1 == 0
	btfss	STATUS, C
	endif
	if	dst == op1
	addwf	op1+2, F
	else
	addwf	op1+2, W
	movwf	dst+2
	endif

byteop = (lit & 0xff000000) >> 24	;Get operand LSB + 3 (MSB)
	movlw	byteop
	btfsc	STATUS, C		;See LSB + 1 for comments
	movlw	byteop+1
	if	byteop+1 == 0
	btfss	STATUS, C
	endif
	if	dst == op1
	addwf	op1+3, F
	else
	addwf	op1+3, W
	movwf	dst+3
	endif

	endm


;Subtract the literal 32 bit value in lit from the op1 area and put
;the result in dst.

;The macro adjust itself to save space if dst is the same location
;of the op1 operand (i.e. op1 = op1 - lit).

msubl32	macro	op1, op2, dst

	maddl32	op1, -op2, dst		;Perform subtraction using 2's complement

	endm


	endif				;_math32_inc_

